package cn.com.qmth.examcloud.core.examwork.api.controller;

import static org.springframework.data.domain.ExampleMatcher.GenericPropertyMatchers.contains;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Example;
import org.springframework.data.domain.ExampleMatcher;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.security.bean.User;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.CourseGroupBean;
import cn.com.qmth.examcloud.core.examwork.dao.CourseGroupRelationRepo;
import cn.com.qmth.examcloud.core.examwork.dao.CourseGroupRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamOrgRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.CourseGroupEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.CourseGroupRelation;
import cn.com.qmth.examcloud.core.examwork.dao.entity.Exam;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamOrgEntity;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudent;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamService;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamStudentService;
import io.swagger.annotations.ApiOperation;

/**
 * 考试服务API Created by songyue on 17/1/13.
 */
@Transactional
@RestController
@RequestMapping("${$rmp.ctr.examwork}/exam")
public class ExamController extends ControllerSupport {

	@Autowired
	ExamRepo examRepo;

	@Autowired
	ExamService examService;

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	CourseGroupRepo courseGroupRepo;

	@Autowired
	ExamOrgRepo examOrgRepo;

	@Autowired
	CourseGroupRelationRepo courseGroupRelationRepo;

	@ApiOperation(value = "查询所有考试批次", notes = "分页带查询")
	@GetMapping("all/{curPage}/{pageSize}")
	public Page<Exam> getAllExam(HttpServletRequest request, @ModelAttribute Exam examCriteria,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {
		User accessUser = getAccessUser();
		examCriteria.setRootOrgId(accessUser.getRootOrgId());
		Page<Exam> page = examService.getAllExam(examCriteria,
				new PageRequest(curPage, pageSize, new Sort(Direction.DESC, "updateTime")));
		return page;
	}

	@ApiOperation(value = "查询所有考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("all")
	public ResponseEntity getEnableExam(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		User accessUser = getAccessUser();

		examCriteria.setRootOrgId(accessUser.getRootOrgId());
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param name
	 * @return
	 */
	@ApiOperation(value = "查询考试批次(包含有效、无效)")
	@GetMapping("queryByName")
	public List<Exam> queryByName(@RequestParam(required = false) String name) {

		if (StringUtils.isBlank(name)) {
			List<Exam> list = Lists.newArrayList();
			return list;
		}

		Specification<Exam> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), getRootOrgId()));
			predicates.add(cb.like(root.get("name"), toSqlSearchPattern(name)));

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};
		Sort sort = new Sort(Direction.DESC, "updateTime");
		List<Exam> page = examRepo.findAll(specification, sort);

		return page;
	}

	@ApiOperation(value = "根据机构ID和考试类型查询所有有效考试批次", notes = "根据机构ID和考试类型查询所有有效考试批次")
	@GetMapping("rootOrgId/{rootOrgId}/{examType}")
	public ResponseEntity getEnableExamByRootOrgId(@PathVariable Long rootOrgId,
			@PathVariable String examType) {
		Exam examCriteria = new Exam();
		examCriteria.setRootOrgId(rootOrgId);
		if (StringUtils.isNotBlank(examType)) {
			examCriteria.setExamType(ExamType.valueOf(examType));
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.getAllExam(examCriteria), HttpStatus.OK);
	}

	@ApiOperation(value = "查询所有考试批次(包含有效无效)", notes = "不分页带查询")
	@GetMapping("all/both")
	public List<Exam> getAllExam(@ModelAttribute Exam examCriteria) {
		examCriteria.setRootOrgId(getRootOrgId());
		ExampleMatcher exampleMatcher = ExampleMatcher.matching().withMatcher("name", contains());
		Example<Exam> examExample = Example.of(examCriteria, exampleMatcher);
		return examRepo.findAll(examExample, new Sort(Direction.DESC, "id"));
	}

	@ApiOperation(value = "根据名称查询考试批次(包含有效)", notes = "不分页带查询")
	@GetMapping("name")
	public ResponseEntity getEnableExamByName(HttpServletRequest request,
			@ModelAttribute Exam examCriteria) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			examCriteria.setRootOrgId(accessUser.getRootOrgId());
		}
		examCriteria.setEnable(true);
		return new ResponseEntity(examService.findByNameAndRootOrgId(examCriteria.getName(),
				examCriteria.getRootOrgId()), HttpStatus.OK);
	}

	@ApiOperation(value = "按ID查询考试批次", notes = "ID查询")
	@GetMapping("{id}")
	public ResponseEntity<Exam> getExamById(@PathVariable Long id) {
		return new ResponseEntity<Exam>(examService.getExamById(id), HttpStatus.OK);
	}

	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping()
	public Exam addExam(@RequestBody Exam exam) {
		trim(exam);

		if (null == exam.getExamType()) {
			throw new StatusException("E-001005", "考试类型为空");
		}

		if (null != exam.getBeforeExamRemark() && exam.getBeforeExamRemark().length() > 250000) {
			throw new StatusException("E-001002", "考前说明内容过大");
		}
		if (null != exam.getAfterExamRemark() && exam.getAfterExamRemark().length() > 250000) {
			throw new StatusException("E-001002", "考后说明内容过大");
		}
		if (null != exam.getCheatingRemark() && exam.getCheatingRemark().length() > 250000) {
			throw new StatusException("E-001002", "作弊说明内容过大");
		}

		User accessUser = getAccessUser();

		exam.setRootOrgId(accessUser.getRootOrgId());
		exam.setCanStuDel(true);
		if (!examService.checkExamName(exam)) {
			throw new StatusException("E-001001", "考试名称已存在,请重新填写");
		}
		Exam ret = examService.insertExam(exam, accessUser);
		return ret;
	}

	@ApiOperation(value = "更新考试批次", notes = "更新")
	@PutMapping()
	public Exam updateExam(@RequestBody Exam exam) {
		trim(exam);

		if (null == exam.getExamType()) {
			throw new StatusException("E-001005", "考试类型为空");
		}

		if (!examService.checkExamName(exam)) {
			throw new StatusException("E-001001", "考试名称已存在,请重新填写");
		}

		if (null != exam.getBeforeExamRemark() && exam.getBeforeExamRemark().length() > 250000) {
			throw new StatusException("E-001002", "考前说明内容过大");
		}
		if (null != exam.getAfterExamRemark() && exam.getAfterExamRemark().length() > 250000) {
			throw new StatusException("E-001002", "考后说明内容过大");
		}
		if (null != exam.getCheatingRemark() && exam.getCheatingRemark().length() > 250000) {
			throw new StatusException("E-001002", "作弊说明内容过大");
		}

		Exam ret = examService.saveExam(exam);
		return ret;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param id
	 * @return
	 */
	@ApiOperation(value = "按ID删除考试批次", notes = "删除")
	@DeleteMapping("{id}")
	public List<Long> deleteExam(@PathVariable String id) {
		List<Long> examIds = Stream.of(id.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		examService.deleteExam(examIds);
		return examIds;
	}

	@ApiOperation(value = "启用考试", notes = "启用考试")
	@PutMapping("enable/{ids}")
	public ResponseEntity enableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			Exam exam = examRepo.findOne(examId);
			exam.setEnable(true);
			examRepo.save(exam);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "禁用考试", notes = "禁用考试")
	@PutMapping("disable/{ids}")
	public ResponseEntity disableUser(@PathVariable String ids) {
		List<Long> examIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		for (Long examId : examIds) {
			Exam exam = examRepo.findOne(examId);
			exam.setEnable(false);
			examRepo.save(exam);
		}
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "回填考生不可删除标志", notes = "回填考生不可删除标志")
	@PutMapping("canNotDel/{id}")
	public ResponseEntity disableUser(@PathVariable Long id) {
		examRepo.canNotDel(id);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "网考初始化更新考务信息", notes = "网考初始化更新考务信息")
	@PutMapping("updateExamInfo")
	public ResponseEntity updateExamInfo(@RequestParam Long examId, @RequestParam Long examStuId) {
		examRepo.canNotDel(examId);
		examStudentService.updateExamStudentInfo(examStuId);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "网考更新考务信息", notes = "网考更新考务信息")
	@PutMapping("updateExamStudent")
	public ResponseEntity updateExamStudent(@RequestParam Long examId,
			@RequestParam Long examStuId) {
		examRepo.canNotDel(examId);
		ExamStudent examStudent = examStudentRepo.findOne(examStuId);
		examStudent.setFinished(true);
		examStudentRepo.save(examStudent);
		return new ResponseEntity(HttpStatus.OK);
	}

	@ApiOperation(value = "查询课程组集合", notes = "")
	@GetMapping("queryCourseGroupList/{examId}/{curPage}/{pageSize}")
	public List<CourseGroupBean> queryCourseGroupList(@PathVariable Long examId,
			@PathVariable Integer curPage, @PathVariable Integer pageSize) {

		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"updateTime");
		List<CourseGroupEntity> groupList = courseGroupRepo.findAllByExamId(examId, pageable);

		List<CourseGroupBean> ret = Lists.newArrayList();

		for (CourseGroupEntity curCourseGroup : groupList) {
			CourseGroupBean bean = new CourseGroupBean();
			bean.setBeginTime(curCourseGroup.getBeginTime());
			bean.setCreationTime(curCourseGroup.getCreationTime());
			bean.setDescription(curCourseGroup.getDescription());
			bean.setEndTime(curCourseGroup.getEndTime());
			bean.setExamId(curCourseGroup.getExamId());
			bean.setId(curCourseGroup.getId());
			bean.setName(curCourseGroup.getName());
			bean.setUpdateTime(curCourseGroup.getUpdateTime());

			List<Long> courseIdList = Lists.newArrayList();
			bean.setCourseIdList(courseIdList);

			List<CourseGroupRelation> relationList = courseGroupRelationRepo
					.findAllByGroupId(bean.getId());

			for (CourseGroupRelation cur : relationList) {
				courseIdList.add(cur.getCourseId());
			}

			ret.add(bean);
		}

		return ret;
	}

	@ApiOperation(value = "通过课程ID查询课程组集合", notes = "")
	@GetMapping("queryCourseGroupList/{examId}/{courseId}")
	public List<CourseGroupBean> queryCourseGroupList(@PathVariable Long examId,
			@PathVariable Long courseId) {

		List<CourseGroupRelation> relationList = courseGroupRelationRepo
				.findAllByCourseIdAndExamId(courseId, examId);

		List<Long> groupIdList = Lists.newArrayList();
		for (CourseGroupRelation cur : relationList) {
			Long groupId = cur.getGroupId();
			groupIdList.add(groupId);
		}

		List<CourseGroupEntity> groupList = courseGroupRepo
				.findAllByIdInOrderByUpdateTimeDesc(groupIdList);

		List<CourseGroupBean> ret = Lists.newArrayList();

		for (CourseGroupEntity curCourseGroup : groupList) {
			CourseGroupBean bean = new CourseGroupBean();
			bean.setBeginTime(curCourseGroup.getBeginTime());
			bean.setCreationTime(curCourseGroup.getCreationTime());
			bean.setDescription(curCourseGroup.getDescription());
			bean.setEndTime(curCourseGroup.getEndTime());
			bean.setExamId(curCourseGroup.getExamId());
			bean.setId(curCourseGroup.getId());
			bean.setName(curCourseGroup.getName());
			bean.setUpdateTime(curCourseGroup.getUpdateTime());

			List<Long> courseIdList = Lists.newArrayList();
			bean.setCourseIdList(courseIdList);

			List<CourseGroupRelation> curRelationList = courseGroupRelationRepo
					.findAllByGroupId(bean.getId());

			for (CourseGroupRelation cur : curRelationList) {
				courseIdList.add(cur.getCourseId());
			}
		}

		return ret;
	}

	@ApiOperation(value = "更新课程组", notes = "")
	@PutMapping("courseGroup")
	public void updateCourseGroup(@RequestBody CourseGroupBean courseGroupBean,
			HttpServletRequest request) {

		CourseGroupEntity courseGroup = new CourseGroupEntity();
		courseGroup.setBeginTime(courseGroupBean.getBeginTime());
		courseGroup.setCreationTime(courseGroupBean.getCreationTime());
		courseGroup.setDescription(courseGroupBean.getDescription());
		courseGroup.setEndTime(courseGroupBean.getEndTime());
		courseGroup.setExamId(courseGroupBean.getExamId());
		courseGroup.setId(courseGroupBean.getId());
		courseGroup.setName(courseGroupBean.getName());
		courseGroup.setUpdateTime(courseGroupBean.getUpdateTime());

		courseGroupRepo.save(courseGroup);
		List<Long> courseIdList = courseGroupBean.getCourseIdList();

		List<CourseGroupRelation> relationList = Lists.newArrayList();

		courseGroupRelationRepo.deleteByGroupId(courseGroup.getId());

		for (Long cur : courseIdList) {
			CourseGroupRelation relation = new CourseGroupRelation();
			relation.setCourseId(cur);
			relation.setExamId(courseGroupBean.getExamId());
			relation.setGroupId(courseGroup.getId());
			relationList.add(relation);
		}
		courseGroupRelationRepo.save(relationList);
	}

	@ApiOperation(value = "删除课程组", notes = "")
	@DeleteMapping("courseGroup/{id}")
	public void deleteSpecificExam(@PathVariable Long id, HttpServletRequest request) {
		courseGroupRepo.delete(id);
		courseGroupRelationRepo.deleteByGroupId(id);
	}

	@ApiOperation(value = "查询考试相关的学习中心设置", notes = "")
	@GetMapping("getExamOrgList/{examId}/{curPage}/{pageSize}")
	public List<?> getExamOrgList(@PathVariable Long examId, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		Pageable pageable = new PageRequest(curPage - 1, pageSize, Sort.Direction.DESC,
				"updateTime");
		List<ExamOrgEntity> allByExamId = examOrgRepo.findAllByExamId(examId, pageable);
		return allByExamId;
	}

	@ApiOperation(value = "新增考试相关的学习中心设置", notes = "")
	@PostMapping("examOrg")
	public ExamOrgEntity addExamOrg(@RequestBody ExamOrgEntity examOrg) {
		ExamOrgEntity ret = examOrgRepo.save(examOrg);
		return ret;
	}

	@ApiOperation(value = "更新考试相关的学习中心设置", notes = "")
	@PutMapping("examOrg")
	public ExamOrgEntity updateExamOrg(@RequestBody ExamOrgEntity examOrg) {
		ExamOrgEntity ret = examOrgRepo.save(examOrg);
		return ret;
	}

	@ApiOperation(value = "删除考试相关的学习中心设置", notes = "")
	@DeleteMapping("examOrg/{examId}/{orgId}")
	public void deleteExamOrg(@PathVariable Long examId, @PathVariable Long orgId) {
		ExamOrgEntity examOrg = new ExamOrgEntity();
		examOrg.setExamId(examId);
		examOrg.setOrgId(orgId);
		examOrgRepo.delete(examOrg);
	}

}
