package cn.com.qmth.examcloud.core.examwork.api.controller;

import java.math.BigInteger;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;
import java.util.stream.Stream;

import javax.persistence.criteria.Predicate;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.fileupload.disk.DiskFileItem;
import org.apache.commons.lang3.StringUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Sort;
import org.springframework.data.domain.Sort.Direction;
import org.springframework.data.jpa.domain.Specification;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import com.google.common.collect.Lists;

import cn.com.qmth.examcloud.common.dto.core.Course;
import cn.com.qmth.examcloud.common.dto.examwork.CommonExamStudent;
import cn.com.qmth.examcloud.commons.base.util.ErrorMsg;
import cn.com.qmth.examcloud.commons.base.util.excel.ExcelError;
import cn.com.qmth.examcloud.commons.web.helpers.page.PageInfo;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.request.InsertOrUpdateStudentReq;
import cn.com.qmth.examcloud.core.basic.api.response.InsertOrUpdateStudentResp;
import cn.com.qmth.examcloud.core.examwork.api.controller.bean.ExamStudentDomain;
import cn.com.qmth.examcloud.core.examwork.base.util.ExportUtils;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.bean.ExamStudentAssembler;
import cn.com.qmth.examcloud.core.examwork.dao.bean.ExamStudentDTO;
import cn.com.qmth.examcloud.core.examwork.dao.bean.OrgExamInfoDTO;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudent;
import cn.com.qmth.examcloud.core.examwork.service.ExamStudentImportService;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamStudentService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import io.swagger.annotations.ApiOperation;

/**
 * 考生服务API Created by songyue on 17/1/13.
 */
@RestController
@RequestMapping("${$rmp.ctr.examwork}/exam_student")
public class ExamStudentController extends ControllerSupport {

	private static final Logger log = LoggerFactory.getLogger(ExamStudentController.class);

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	ExamStudentAssembler examStudentAssembler;

	@Autowired
	ExamRepo examRepo;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamStudentCloudService examStudentCloudService;

	@Autowired
	ExamStudentImportService examStudentImportService;

	@GetMapping("/query")
	public List<ExamStudent> find(
			@RequestParam(value = "student_id", required = false) Long studentId) {

		return examStudentRepo.findAll((root, query, cb) -> {

			List<Predicate> predicates = new ArrayList<Predicate>();

			if (studentId != null) {
				predicates.add(cb.equal(root.get("studentId"), studentId));
			}

			Predicate[] pre = new Predicate[predicates.size()];
			return query.where(predicates.toArray(pre)).getRestriction();
		});
	}

	/**
	 * 重构
	 *
	 * @author WANGWEI
	 * @param examCriteria
	 * @param curPage
	 * @param pageSize
	 * @return
	 */
	@ApiOperation(value = "查询考试学生带条件和分页", notes = "带条件带分页")
	@GetMapping("/all/{curPage}/{pageSize}")
	public PageInfo<ExamStudentDomain> getExamStudentList(
			@ModelAttribute ExamStudentDomain examCriteria, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		Specification<ExamStudent> specification = (root, query, cb) -> {
			List<Predicate> predicates = new ArrayList<>();
			predicates.add(cb.equal(root.get("rootOrgId"), getRootOrgId()));

			if (null != examCriteria.getOrgId()) {
				predicates.add(cb.equal(root.get("orgId"), examCriteria.getOrgId()));
			}

			if (null != examCriteria.getExamId()) {
				predicates.add(cb.equal(root.get("exam").get("id"), examCriteria.getExamId()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getStudentName())) {
				predicates.add(cb.like(root.get("name"),
						toSqlSearchPattern(examCriteria.getStudentName())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getStudentCode())) {
				predicates.add(cb.like(root.get("studentCode"),
						toSqlSearchPattern(examCriteria.getStudentCode())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseCode())) {
				predicates.add(cb.equal(root.get("courseCode"), examCriteria.getCourseCode()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseLevel())) {
				predicates.add(cb.equal(root.get("courseLevel"), examCriteria.getCourseLevel()));
			}
			if (StringUtils.isNotEmpty(examCriteria.getCourseName())) {
				predicates.add(cb.like(root.get("courseName"),
						toSqlSearchPattern(examCriteria.getCourseName())));
			}
			if (!StringUtils.isEmpty(examCriteria.getExamSite())) {
				predicates.add(cb.like(root.get("examSite"),
						toSqlSearchPattern(examCriteria.getExamSite())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getIdentityNumber())) {
				predicates.add(cb.like(root.get("identityNumber"),
						toSqlSearchPattern(examCriteria.getIdentityNumber())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getSpecialtyName())) {
				predicates.add(cb.like(root.get("specialtyName"),
						toSqlSearchPattern(examCriteria.getSpecialtyName())));
			}
			if (StringUtils.isNotEmpty(examCriteria.getInfoCollector())) {
				predicates.add(cb.like(root.get("infoCollector"),
						toSqlSearchPattern(examCriteria.getInfoCollector())));
			}

			return cb.and(predicates.toArray(new Predicate[predicates.size()]));
		};

		PageRequest pageRequest = new PageRequest(curPage, pageSize,
				new Sort(Direction.DESC, "updateTime"));

		Page<ExamStudent> examStudents = examStudentRepo.findAll(specification, pageRequest);

		List<ExamStudentDomain> ret = Lists.newArrayList();

		for (ExamStudent cur : examStudents) {
			ExamStudentDomain bean = new ExamStudentDomain();
			bean.setId(cur.getId());
			bean.setExamId(cur.getExam().getId());
			bean.setExamName(cur.getExam().getName());
			bean.setStudentId(cur.getStudentId());
			bean.setStudentName(cur.getName());
			bean.setStudentCode(cur.getStudentCode());
			bean.setIdentityNumber(cur.getIdentityNumber());
			bean.setCourseCode(cur.getCourseCode());
			bean.setCourseName(cur.getCourseName());
			bean.setInfoCollector(cur.getInfoCollector());
			bean.setOrgId(cur.getOrgId());
			bean.setOrgCode(cur.getOrgCode());
			bean.setOrgName(cur.getOrgName());
			bean.setPaperType(cur.getPaperType());
			bean.setPhone(cur.getPhone());
			bean.setGrade(cur.getGrade());
			bean.setSpecialtyName(cur.getSpecialtyName());
			bean.setExamSite(cur.getExamSite());
			bean.setFinished(cur.getFinished());
			bean.setExamType(cur.getExam().getExamType().name());
			ret.add(bean);
		}
		return new PageInfo<ExamStudentDomain>(examStudents, ret);
	}

	/**
	 * 
	 * @return
	 */
	@ApiOperation(value = "查询重考考生", notes = "查询重考考生")
	@PostMapping("/reexamineStudents/{curPage}/{pageSize}")
	public ResponseEntity getReexamineStudents(
			@RequestBody CommonExamStudent examStudentSpecification, @PathVariable Integer curPage,
			@PathVariable Integer pageSize) {
		try {
			Page<ExamStudent> pageExamStudents = examStudentService
					.getReexamineStudent(examStudentSpecification, curPage, pageSize);
			if (pageExamStudents == null || pageExamStudents.getContent() == null
					|| pageExamStudents.getContent().size() == 0) {
				return new ResponseEntity(new ErrorMsg("无数据"), HttpStatus.OK);
			}
			return new ResponseEntity(pageExamStudents, HttpStatus.OK);
		} catch (Exception e) {
			log.error("调用失败:", e);
			return new ResponseEntity(new ErrorMsg("调用失败"), HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "查询所有考试学生带条件", notes = "带条件不分页")
	@GetMapping("/all")
	public ResponseEntity getAllExamStudent(HttpServletRequest request,
			@ModelAttribute ExamStudentDTO examStudent) {
		cn.com.qmth.examcloud.commons.web.security.bean.User accessUser = getAccessUser();
		if (accessUser != null) {
			if (accessUser.getRootOrgId().longValue() == accessUser.getOrgId().longValue()
					|| examStudent.getStudentId() != null) {
				examStudent.setRootOrgId(accessUser.getRootOrgId());
			} else {
				examStudent.setOrgId(accessUser.getOrgId());
			}
		}
		return new ResponseEntity(examStudentService.getAllExamStudent(examStudent), HttpStatus.OK);
	}

	@ApiOperation(value = "查询所有考试学生带条件", notes = "带条件按照限制查询")
	@GetMapping("/limit/{examId}/{startLimit}/{endLimit}")
	public ResponseEntity getExamStudentByLimit(@PathVariable("examId") Long examId,
			@PathVariable("startLimit") Integer startLimit,
			@PathVariable("endLimit") Integer endLimit) {
		List<ExamStudent> examStudents = examStudentService.getExamStudentByLimit(examId,
				startLimit, endLimit);
		return new ResponseEntity(examStudents, HttpStatus.OK);
	}

	@ApiOperation(value = "按ID查询考试学生", notes = "ID查询")
	@GetMapping("/{id}")
	public ResponseEntity getExamStudentById(@PathVariable Long id) {
		return new ResponseEntity(examStudentService.findById(id), HttpStatus.OK);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param request
	 * @param examStudent
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "新增考试学生", notes = "新增")
	@PostMapping()
	public ExamStudentDomain addExamStudent(HttpServletRequest request,
			@RequestBody ExamStudentDomain examStudent) {
		trim(examStudent);
		Long rootOrgId = getRootOrgId();
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setInfoCollector(examStudent.getInfoCollector());
		return saveExamStudent(examStudent);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "更新考试学生", notes = "更新")
	@PutMapping()
	public ExamStudentDomain updateExamStudent(@RequestBody ExamStudentDomain examStudent) {
		trim(examStudent);
		Long rootOrgId = getRootOrgId();
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setInfoCollector(examStudent.getInfoCollector());
		return saveExamStudent(examStudent);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examStudent
	 * @return
	 */
	private ExamStudentDomain saveExamStudent(ExamStudentDomain examStudent) {
		InsertOrUpdateStudentReq saveStudentReq = new InsertOrUpdateStudentReq();
		saveStudentReq.setIdentityNumber(examStudent.getIdentityNumber());
		saveStudentReq.setName(examStudent.getStudentName());
		saveStudentReq.setPhoneNumber(examStudent.getPhone());
		saveStudentReq.setOrgCode(examStudent.getOrgCode());
		saveStudentReq.setOrgId(examStudent.getOrgId());
		saveStudentReq.setOrgName(examStudent.getOrgName());
		saveStudentReq.setRemark(examStudent.getRemark());
		saveStudentReq.setRootOrgId(examStudent.getRootOrgId());
		saveStudentReq.setStudentCode(examStudent.getStudentCode());

		InsertOrUpdateStudentResp insertOrUpdateStudentResp = studentCloudService
				.insertOrUpdateStudent(saveStudentReq);

		Long studentId = insertOrUpdateStudentResp.getStudentId();

		SaveExamStudentReq saveExamStudentReq = new SaveExamStudentReq();
		saveExamStudentReq.setCourseCode(examStudent.getCourseCode());
		saveExamStudentReq.setCourseLevel(examStudent.getCourseLevel());
		saveExamStudentReq.setCourseName(examStudent.getCourseName());
		saveExamStudentReq.setExamId(examStudent.getExamId());
		saveExamStudentReq.setIdentityNumber(examStudent.getIdentityNumber());
		saveExamStudentReq.setPaperType(examStudent.getPaperType());
		saveExamStudentReq.setRootOrgId(examStudent.getRootOrgId());
		saveExamStudentReq.setStudentCode(examStudent.getStudentCode());
		saveExamStudentReq.setStudentName(examStudent.getStudentName());
		saveExamStudentReq.setStudentId(studentId);
		saveExamStudentReq.setInfoCollector(examStudent.getInfoCollector());
		saveExamStudentReq.setGrade(examStudent.getGrade());
		saveExamStudentReq.setExamSite(examStudent.getExamSite());
		saveExamStudentReq.setSpecialtyName(examStudent.getSpecialtyName());

		SaveExamStudentResp saveExamStudentResp = examStudentCloudService
				.saveExamStudent(saveExamStudentReq);
		ExamStudentBean examStudentBean = saveExamStudentResp.getExamStudentBean();

		ExamStudentDomain ret = new ExamStudentDomain();
		ret.setId(examStudentBean.getId());
		ret.setStudentId(studentId);
		ret.setStudentCode(examStudentBean.getStudentCode());
		ret.setStudentName(examStudentBean.getStudentName());
		ret.setCourseCode(examStudentBean.getCourseCode());
		ret.setCourseName(examStudentBean.getCourseName());
		return ret;
	}

	@ApiOperation(value = "更新考试学生缺考状态", notes = "更新缺考")
	@Transactional
	@PutMapping("/{id}")
	public ResponseEntity<ExamStudent> getExamStudentById(@PathVariable Long id,
			@RequestParam boolean finished) {
		ExamStudent examStudent = examStudentRepo.findOne(id);
		examStudent.setFinished(finished);
		// 进入考试时，判断考试次数是否小于考试信息(ecs_exam)中的考试次数，如果是，考试次数加一
		long examTimes = examStudent.getExam().getExamTimes();
		if (examTimes > examStudent.getNormalExamTimes()) {
			examStudent.setNormalExamTimes(examStudent.getNormalExamTimes() + 1);
		}
		return new ResponseEntity<ExamStudent>(examStudentRepo.save(examStudent), HttpStatus.OK);
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param ids
	 * @return
	 */
	@Transactional
	@ApiOperation(value = "按ID删除考试学生", notes = "删除")
	@DeleteMapping("/{ids}")
	public List<Long> deleteExamStudent(@PathVariable String ids) {
		List<Long> examStuIds = Stream.of(ids.split(",")).map(s -> Long.parseLong(s.trim()))
				.collect(Collectors.toList());
		examStudentService.deleteExamStudent(examStuIds);
		return examStuIds;
	}

	/**
	 * 方法注释
	 *
	 * @author WANGWEI
	 * @param examId
	 * @return
	 */
	@ApiOperation(value = "按考试删除考试学生", notes = "按考试删除")
	@Transactional
	@DeleteMapping("/exam/{examId}")
	public Long deleteExamStudents(@PathVariable Long examId) {
		examStudentService.deleteExamStudents(examId);
		return examId;
	}

	/**
	 * 重构
	 *
	 * @author WANGWEI
	 * @param examId
	 * @param file
	 * @return
	 * @throws Exception
	 */
	@ApiOperation(value = "导入考试学生", notes = "导入")
	@PostMapping("/import")
	public List<ExcelError> importExamStudent(@RequestParam Long examId,
			@RequestParam CommonsMultipartFile file) throws Exception {

		DiskFileItem item = (DiskFileItem) file.getFileItem();
		examStudentImportService.importExamStudent(item.getStoreLocation(),
				file.getOriginalFilename(), getRootOrgId(), examId);
		return null;
	}

	@ApiOperation(value = "下载导入模板", notes = "下载导入模板")
	@GetMapping("/download")
	public void importFileTemplate(HttpServletResponse response) {
		List<ExamStudentDTO> list = new ArrayList<ExamStudentDTO>();
		ExportUtils.exportEXCEL("考生导入模板", ExamStudentDTO.class, list, response);
	}

	@ApiOperation(value = "照片检验", notes = "检验")
	@PostMapping("/photo_check")
	public ResponseEntity photoCheck(@RequestParam Long examId) {
		try {
			List<ErrorMsg> errorMsgs = examStudentService.photoCheck(examId);
			return new ResponseEntity(errorMsgs, HttpStatus.OK);
		} catch (Exception e) {
			e.printStackTrace();
			return new ResponseEntity(HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "导出考试学生带条件", notes = "导出")
	@GetMapping("/export")
	public void exportExamStudent(@ModelAttribute ExamStudentDTO examCriteria,
			HttpServletResponse response) {
		List<ExamStudentDTO> list = new ArrayList<ExamStudentDTO>();
		examStudentService.getAllExamStudent(examCriteria).forEach(c -> {
			list.add(examStudentAssembler.toDTO(c));
		});
		ExportUtils.exportEXCEL("课程列表", ExamStudentDTO.class, list, response);

	}

	@ApiOperation(value = "复制考试学生", notes = "复制")
	@Transactional
	@PostMapping("/copy/{sourceExamId}/{targetExamId}")
	public void copyExamStudent(@PathVariable Long sourceExamId, @PathVariable Long targetExamId) {
		examStudentService.copyExamStudent(sourceExamId, targetExamId);
	}

	@ApiOperation(value = "考生完成数量", notes = "考生完成数量")
	@GetMapping("/count-by-finished")
	public Map<String, Integer> countByFinished(@RequestParam Long examId) {

		int finished = examStudentRepo.countByExamIdAndFinished(examId, true);
		int unFinished = examStudentRepo.countByExamIdAndFinished(examId, false);
		Map<String, Integer> result = new HashMap<>();
		result.put("finished", finished);
		result.put("unFinished", unFinished);
		return result;
	}

	@ApiOperation(value = "按学习中心统计完成数量", notes = "按学习中心统计完成数量")
	@GetMapping("/count-by-campus")
	public List<Map<String, Object>> countByCampus(@RequestParam Long examId) {

		List<Object[]> resultList = examStudentRepo.countByCampusAndFinished(examId);
		List<Map<String, Object>> returnList = new ArrayList<>();
		for (Object[] objects : resultList) {
			Long orgId = ((BigInteger) objects[0]).longValue();
			String orgName = (String) objects[1];
			Boolean finished = (Boolean) objects[2];
			Integer count = ((BigInteger) objects[3]).intValue();

			boolean found = false;
			for (Map<String, Object> m : returnList) {
				if (m.get("orgId").equals(orgId)) {
					found = true;

					if (finished) {
						m.put("finished", count);
					} else {
						m.put("unFinished", count);
					}

					break;
				}
			}
			if (!found) {
				Map<String, Object> map = new HashMap<>();

				map.put("orgId", orgId);
				map.put("orgName", orgName);
				if (finished) {
					map.put("finished", count);
				} else {
					map.put("unFinished", count);
				}

				returnList.add(map);
			}

		}

		return returnList;

	}

	@ApiOperation(value = "学习中心考试进度信息", notes = "学习中心考试进度信息")
	@GetMapping("/findOrgExamInfos")
	public ResponseEntity<Object> findOrgExamInfos(String examId, String orgCode) {
		try {
			List<OrgExamInfoDTO> orgExamInfoDTOs = examStudentService.findOrgExamInfos(examId,
					orgCode);
			return new ResponseEntity<Object>(orgExamInfoDTOs, HttpStatus.OK);
		} catch (Exception e) {
			e.printStackTrace();
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "从考试表中查询课程", notes = "从考试表中查询课程")
	@GetMapping("/findCoursesFromExamStudent")
	public ResponseEntity<Object> findCoursesFromExamStudent(String examId, String orgId) {
		try {
			List<Course> courses = examStudentService.findCoursesFromExamStudent(examId, orgId);
			return new ResponseEntity<Object>(courses, HttpStatus.OK);
		} catch (Exception e) {
			e.printStackTrace();
			return new ResponseEntity(new ErrorMsg(e.getMessage()),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
	}

	@ApiOperation(value = "设置考生为重考状态", notes = "设置考生为重考状态")
	@Transactional
	@PostMapping("/setExamStudentReexamine")
	public ResponseEntity<Object> setExamStudentIsReexamine(
			@RequestBody CommonExamStudent commonExamStudent) {
		if (commonExamStudent.getId() == null) {
			return new ResponseEntity<Object>(new ErrorMsg("考生ID不能为空"),
					HttpStatus.INTERNAL_SERVER_ERROR);
		}
		examStudentService.setExamStudentIsReexamine(commonExamStudent);
		return new ResponseEntity<Object>(HttpStatus.OK);
	}
}
