package cn.com.qmth.examcloud.core.examwork.api.provider;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.examwork.dao.entity.Exam;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamService;
import cn.com.qmth.examcloud.examwork.api.ExamCloudService;
import cn.com.qmth.examcloud.examwork.api.request.GetExamReq;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamReq;
import cn.com.qmth.examcloud.examwork.api.response.GetExamResp;
import io.swagger.annotations.ApiOperation;

/**
 * @author chenken
 * @date 2018年5月3日 下午2:08:59
 * @company QMTH
 * @description ExamProvider.java
 */
@RestController
@RequestMapping("${$rmp.cloud.examwork}" + "exam")
public class ExamCloudServiceProvider extends ControllerSupport implements ExamCloudService {

	private static final long serialVersionUID = 6850508158980856785L;

	@Autowired
	private ExamService examService;

	@ApiOperation(value = "新增考试批次", notes = "新增")
	@PostMapping
	@Override
	public void saveExam(@RequestBody SaveExamReq req) {
		Exam exam = new Exam();
		exam.setName(req.getName());
		exam.setRemark(req.getRemark());
		exam.setDuration(req.getDuration());
		exam.setExamType(ExamType.valueOf(req.getExamType()));
		exam.setBeginTime(req.getBeginTime());
		exam.setEndTime(req.getEndTime());
		exam.setRootOrgId(req.getRootOrgId());
		exam.setCanStuDel(true);
		if (!examService.checkExamName(exam)) {
			throw new StatusException("E-002001", "考试名称已存在,请重新填写");
		}
		examService.insertExam(exam, null);
	}

	@ApiOperation(value = "根据名称和机构ID查询考试", notes = "根据名称和机构ID查询考试")
	@PostMapping("/findExam")
	@Override
	public GetExamResp getExam(@RequestBody GetExamReq req) {
		Exam exam = examService.findExamByNameAndRootOrgId(req.getName(), req.getRootOrgId());
		GetExamResp examResp = new GetExamResp();
		examResp.setId(exam.getId());
		examResp.setName(exam.getName());
		return examResp;
	}

}
