package cn.com.qmth.examcloud.core.examwork.api.provider;

import java.util.List;

import javax.transaction.Transactional;

import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import cn.com.qmth.examcloud.commons.base.exception.StatusException;
import cn.com.qmth.examcloud.commons.web.support.ControllerSupport;
import cn.com.qmth.examcloud.core.basic.api.CourseCloudService;
import cn.com.qmth.examcloud.core.basic.api.OrgCloudService;
import cn.com.qmth.examcloud.core.basic.api.StudentCloudService;
import cn.com.qmth.examcloud.core.basic.api.bean.CourseBean;
import cn.com.qmth.examcloud.core.basic.api.bean.OrgBean;
import cn.com.qmth.examcloud.core.basic.api.bean.StudentBean;
import cn.com.qmth.examcloud.core.basic.api.request.GetOrgReq;
import cn.com.qmth.examcloud.core.basic.api.request.GetStudentReq;
import cn.com.qmth.examcloud.core.basic.api.request.SaveCourseReq;
import cn.com.qmth.examcloud.core.basic.api.response.GetOrgResp;
import cn.com.qmth.examcloud.core.basic.api.response.GetStudentResp;
import cn.com.qmth.examcloud.core.basic.api.response.SaveCourseResp;
import cn.com.qmth.examcloud.core.examwork.dao.ExamRepo;
import cn.com.qmth.examcloud.core.examwork.dao.ExamStudentRepo;
import cn.com.qmth.examcloud.core.examwork.dao.entity.Exam;
import cn.com.qmth.examcloud.core.examwork.dao.entity.ExamStudent;
import cn.com.qmth.examcloud.core.examwork.service.impl.ExamStudentService;
import cn.com.qmth.examcloud.examwork.api.ExamStudentCloudService;
import cn.com.qmth.examcloud.examwork.api.bean.ExamStudentBean;
import cn.com.qmth.examcloud.examwork.api.request.SaveExamStudentReq;
import cn.com.qmth.examcloud.examwork.api.response.SaveExamStudentResp;
import io.swagger.annotations.ApiOperation;

/**
 * 
 * @author chenken
 * @date 2018年5月4日 下午2:06:14
 * @company QMTH
 * @description ExamStudentProvider.java
 */
@Transactional
@RestController
@RequestMapping("${$rmp.cloud.examwork}" + "examStudent")
public class ExamStudentCloudServiceProvider extends ControllerSupport
		implements
			ExamStudentCloudService {

	@Autowired
	ExamRepo examRepo;

	@Autowired
	OrgCloudService orgCloudService;

	@Autowired
	ExamStudentService examStudentService;

	@Autowired
	StudentCloudService studentCloudService;

	@Autowired
	ExamStudentRepo examStudentRepo;

	@Autowired
	CourseCloudService courseCloudService;

	@ApiOperation(value = "保存考生")
	@PostMapping("saveExamStudent")
	@Override
	public SaveExamStudentResp saveExamStudent(@RequestBody SaveExamStudentReq req) {
		trim(req);
		String paperType = req.getPaperType();
		if (StringUtils.isBlank(paperType)) {
			paperType = "A";
		} else if (!paperType.matches("[A-Z]")) {
			throw new StatusException("E-100020", "paperType must be one of A-Z");
		}

		Long rootOrgId = req.getRootOrgId();
		GetOrgReq getOrgReq = new GetOrgReq();
		getOrgReq.setOrgId(rootOrgId);
		GetOrgResp getOrgResp = orgCloudService.getOrg(getOrgReq);
		OrgBean rootOrg = getOrgResp.getOrg();
		if (null != rootOrg.getParentId()) {
			throw new StatusException("E-100001", "rootOrgId is wrong");
		}

		Exam exam = null;
		Long examId = req.getExamId();
		if (null == examId) {
			String examName = req.getExamName();
			if (StringUtils.isBlank(examName)) {
				throw new StatusException("E-100002", "examId & examName are both blank");
			}
			List<Exam> examList = examRepo.findByNameAndRootOrgId(examName, rootOrgId);
			if (CollectionUtils.isEmpty(examList)) {
				throw new StatusException("E-100005", "考试不存在");
			}
			if (1 < examList.size()) {
				throw new StatusException("E-100005", "考试名称对应多个考试");
			}
			exam = examList.get(0);
		} else {
			exam = examRepo.findOne(examId);
		}

		String studentCode = req.getStudentCode();
		String identityNumber = req.getIdentityNumber();

		if (StringUtils.isBlank(identityNumber)) {
			throw new StatusException("E-100003", "identityNumber is null");
		}
		GetStudentReq getStudentReq = new GetStudentReq();
		getStudentReq.setRootOrgId(rootOrgId);
		getStudentReq.setStudentCode(studentCode);
		getStudentReq.setIdentityNumber(identityNumber);
		GetStudentResp getStudentResp = studentCloudService.getStudent(getStudentReq);

		StudentBean studentInfo = getStudentResp.getStudentInfo();

		String courseCode = req.getCourseCode();
		String courseName = req.getCourseName();
		String courseLevel = req.getCourseLevel();

		SaveCourseReq saveCourseReq = new SaveCourseReq();
		saveCourseReq.setCourseCode(courseCode);
		saveCourseReq.setCourseName(courseName);
		saveCourseReq.setCourseLevel(courseLevel);
		saveCourseReq.setRootOrgId(rootOrgId);

		SaveCourseResp saveCourseResp = courseCloudService.saveCourse(saveCourseReq);
		CourseBean courseBean = saveCourseResp.getCourseBean();

		List<ExamStudent> examStudentList = examStudentRepo
				.findByExamAndStudentIdAndCourseCodeOrderByUpdateTimeDesc(exam, studentInfo.getId(),
						courseBean.getCode());

		ExamStudent examStudent = new ExamStudent();

		if (1 < examStudentList.size()) {
			// 数据需要修正
			throw new StatusException("E-100008", "考生数据重复. studentId: " + studentInfo.getId());
		} else if (1 == examStudentList.size()) {
			examStudent = examStudentList.get(0);
		} else {
			examStudent = new ExamStudent();
			examStudent.setNormalExamTimes(0);
			examStudent.setFinished(false);
			examStudent.setIsReexamine(false);
			examStudent.setCanUpload(false);
		}
		examStudent.setInfoCollector(req.getInfoCollector());
		examStudent.setName(studentInfo.getName());
		examStudent.setRootOrgId(rootOrgId);
		examStudent.setCourseCode(courseCode);
		examStudent.setCourseName(courseName);
		examStudent.setCourseLevel(courseBean.getLevel());
		examStudent.setExam(exam);
		examStudent.setIdentityNumber(studentInfo.getIdentityNumber());
		examStudent.setStudentCode(studentInfo.getStudentCode());
		examStudent.setPaperType(paperType);
		examStudent.setStudentId(studentInfo.getId());
		examStudent.setOrgId(studentInfo.getOrgId());
		examStudent.setOrgName(studentInfo.getOrgName());
		examStudent.setOrgCode(studentInfo.getOrgCode());
		examStudent.setPhone(studentInfo.getPhoneNumber());
		examStudent.setGrade(req.getGrade());
		examStudent.setSpecialtyName(req.getSpecialtyName());
		examStudent.setExamSite(req.getExamSite());
		ExamStudent saved = examStudentRepo.save(examStudent);

		SaveExamStudentResp resp = new SaveExamStudentResp();

		ExamStudentBean examStudentBean = new ExamStudentBean();
		examStudentBean.setId(saved.getId());
		examStudentBean.setCourseCode(saved.getCourseCode());
		examStudentBean.setCourseLevel(saved.getCourseLevel());
		examStudentBean.setCourseName(saved.getCourseName());
		examStudentBean.setExamId(saved.getExam().getId());
		examStudentBean.setExamName(saved.getExam().getName());
		examStudentBean.setIdentityNumber(saved.getIdentityNumber());
		examStudentBean.setStudentCode(saved.getStudentCode());
		examStudentBean.setPaperType(saved.getPaperType());
		examStudentBean.setRootOrgId(saved.getRootOrgId());
		examStudentBean.setStudentName(studentInfo.getName());

		resp.setExamStudentBean(examStudentBean);

		return resp;
	}

}
