package cn.com.qmth.examcloud.core.examwork.dao.entity;

import java.util.Date;

import javax.persistence.Entity;
import javax.persistence.EnumType;
import javax.persistence.Enumerated;
import javax.persistence.GeneratedValue;
import javax.persistence.Id;
import javax.persistence.Lob;
import javax.persistence.Table;
import javax.validation.constraints.NotNull;

import org.springframework.format.annotation.DateTimeFormat;

import cn.com.qmth.examcloud.commons.web.jpa.JpaEntity;
import cn.com.qmth.examcloud.core.examwork.dao.enums.ExamType;
import cn.com.qmth.examcloud.core.examwork.dao.enums.MarkingType;
import cn.com.qmth.examcloud.core.examwork.dao.enums.PracticeType;

/**
 * Created by songyue on 17/1/13.
 */
@Entity
@Table(name = "ecs_exam")
public class Exam extends JpaEntity {

	private static final long serialVersionUID = 4009839764353162256L;

	@Id
	@GeneratedValue
	private Long id;

	/**
	 * 顶级机构Id
	 */
	private Long rootOrgId;

	/**
	 * 考试批次开始时间
	 */
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date beginTime;

	/**
	 * 考试批次结束时间
	 */
	@DateTimeFormat(pattern = "yyyy-MM-dd HH:mm:ss")
	private Date endTime;

	/**
	 * 考试名称
	 */
	@NotNull
	private String name;

	/**
	 * 考试类型
	 */
	@Enumerated(EnumType.STRING)
	private ExamType examType;

	/**
	 * 是否入学考试
	 */
	private Boolean isEntranceExam;

	/**
	 * 考试时长
	 */
	private Integer duration;

	/**
	 * 冻结时间
	 */
	private Integer freezeTime;

	/**
	 * 考试状态
	 */
	private String status;

	private Boolean enable;

	/**
	 * 考试备注
	 */
	private String remark;

	/**
	 * 考试次数
	 */
	private Long examTimes;

	/**
	 * 断点续考时间(秒)
	 */
	private Long examReconnectTime;

	/**
	 * 考前说明
	 */
	@Lob
	private String beforeExamRemark;

	/**
	 * 考后说明
	 */
	@Lob
	private String afterExamRemark;

	/**
	 * 作弊说明
	 */
	@Lob
	private String cheatingRemark;

	/**
	 * 是否展示作弊
	 */
	private Boolean showCheatingRemark;

	/**
	 * 是否显示成绩
	 */
	private String isObjScoreView;

	/**
	 * 练习模式
	 */
	@Enumerated(EnumType.STRING)
	private PracticeType practiceType;

	/**
	 * 单选题补充说明是否可填
	 */
	private Boolean singleEdit;

	/**
	 * 多选题补充说明是否可填
	 */
	private Boolean mutipleEdit;

	/**
	 * 判断题补充说明是否可填
	 */
	private Boolean boolEdit;

	/**
	 * 填空题补充说明是否可填
	 */
	private Boolean fillBlankEdit;

	/**
	 * 单选题补充说明
	 */
	private String singleAnswerRemark;

	/**
	 * 多选题补充说明
	 */
	private String mutipleAnswerRemark;

	/**
	 * 判断题补充说明
	 */
	private String boolAnswerRemark;

	/**
	 * 填空题补充说明
	 */
	private String fillBlankRemark;

	/**
	 * 问答题补充说明
	 */
	private String textAnswerRemark;

	/**
	 * 套题补充说明
	 */
	private String nestedAnswerRemark;

	/**
	 * 是否启用人脸识别
	 */
	private String isFaceEnable;

	/**
	 * 进入考试是否验证人脸识别(强制、非强制)
	 */
	private String isFaceCheck;

	/**
	 * 抓拍间隔
	 */
	private Double snapshotInterval;

	/**
	 * 预警阈值
	 */
	private Double warnThreshold;

	/**
	 * 阅卷方式
	 */
	@Enumerated(EnumType.STRING)
	private MarkingType markingType;

	/**
	 * 学生是否可以删除
	 */
	@NotNull
	private Boolean canStuDel;

	/**
	 * 是否开启人脸活体检测
	 */
	private Boolean isFaceVerify;

	/**
	 * 活体检测开始分钟数
	 */
	private Integer faceVerifyStartMinute;

	/**
	 * 活体检测结束分钟数
	 */
	private Integer faceVerifyEndMinute;

	public Long getId() {
		return id;
	}

	public void setId(Long id) {
		this.id = id;
	}

	public Long getRootOrgId() {
		return rootOrgId;
	}

	public void setRootOrgId(Long rootOrgId) {
		this.rootOrgId = rootOrgId;
	}

	public Date getBeginTime() {
		return beginTime;
	}

	public void setBeginTime(Date beginTime) {
		this.beginTime = beginTime;
	}

	public Date getEndTime() {
		return endTime;
	}

	public void setEndTime(Date endTime) {
		this.endTime = endTime;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public ExamType getExamType() {
		return examType;
	}

	public void setExamType(ExamType examType) {
		this.examType = examType;
	}

	public Integer getDuration() {
		return duration;
	}

	public void setDuration(Integer duration) {
		this.duration = duration;
	}

	public Integer getFreezeTime() {
		return freezeTime;
	}

	public void setFreezeTime(Integer freezeTime) {
		this.freezeTime = freezeTime;
	}

	public String getStatus() {
		return status;
	}

	public void setStatus(String status) {
		this.status = status;
	}

	public String getRemark() {
		return remark;
	}

	public void setRemark(String remark) {
		this.remark = remark;
	}

	public Long getExamTimes() {
		return examTimes;
	}

	public void setExamTimes(Long examTimes) {
		this.examTimes = examTimes;
	}

	public Long getExamReconnectTime() {
		return examReconnectTime;
	}

	public void setExamReconnectTime(Long examReconnectTime) {
		this.examReconnectTime = examReconnectTime;
	}

	public String getBeforeExamRemark() {
		return beforeExamRemark;
	}

	public void setBeforeExamRemark(String beforeExamRemark) {
		this.beforeExamRemark = beforeExamRemark;
	}

	public String getAfterExamRemark() {
		return afterExamRemark;
	}

	public void setAfterExamRemark(String afterExamRemark) {
		this.afterExamRemark = afterExamRemark;
	}

	public String getIsObjScoreView() {
		return isObjScoreView;
	}

	public void setIsObjScoreView(String isObjScoreView) {
		this.isObjScoreView = isObjScoreView;
	}

	public PracticeType getPracticeType() {
		return practiceType;
	}

	public void setPracticeType(PracticeType practiceType) {
		this.practiceType = practiceType;
	}

	public String getSingleAnswerRemark() {
		return singleAnswerRemark;
	}

	public void setSingleAnswerRemark(String singleAnswerRemark) {
		this.singleAnswerRemark = singleAnswerRemark;
	}

	public String getMutipleAnswerRemark() {
		return mutipleAnswerRemark;
	}

	public void setMutipleAnswerRemark(String mutipleAnswerRemark) {
		this.mutipleAnswerRemark = mutipleAnswerRemark;
	}

	public String getBoolAnswerRemark() {
		return boolAnswerRemark;
	}

	public void setBoolAnswerRemark(String boolAnswerRemark) {
		this.boolAnswerRemark = boolAnswerRemark;
	}

	public String getFillBlankRemark() {
		return fillBlankRemark;
	}

	public void setFillBlankRemark(String fillBlankRemark) {
		this.fillBlankRemark = fillBlankRemark;
	}

	public String getTextAnswerRemark() {
		return textAnswerRemark;
	}

	public void setTextAnswerRemark(String textAnswerRemark) {
		this.textAnswerRemark = textAnswerRemark;
	}

	public String getNestedAnswerRemark() {
		return nestedAnswerRemark;
	}

	public void setNestedAnswerRemark(String nestedAnswerRemark) {
		this.nestedAnswerRemark = nestedAnswerRemark;
	}

	public String getIsFaceEnable() {
		return isFaceEnable;
	}

	public void setIsFaceEnable(String isFaceEnable) {
		this.isFaceEnable = isFaceEnable;
	}

	public String getIsFaceCheck() {
		return isFaceCheck;
	}

	public void setIsFaceCheck(String isFaceCheck) {
		this.isFaceCheck = isFaceCheck;
	}

	public Double getSnapshotInterval() {
		return snapshotInterval;
	}

	public void setSnapshotInterval(Double snapshotInterval) {
		this.snapshotInterval = snapshotInterval;
	}

	public Double getWarnThreshold() {
		return warnThreshold;
	}

	public void setWarnThreshold(Double warnThreshold) {
		this.warnThreshold = warnThreshold;
	}

	public MarkingType getMarkingType() {
		return markingType;
	}

	public void setMarkingType(MarkingType markingType) {
		this.markingType = markingType;
	}

	public Boolean getEnable() {
		return enable;
	}

	public void setEnable(Boolean enable) {
		this.enable = enable;
	}

	public Boolean getSingleEdit() {
		return singleEdit;
	}

	public void setSingleEdit(Boolean singleEdit) {
		this.singleEdit = singleEdit;
	}

	public Boolean getMutipleEdit() {
		return mutipleEdit;
	}

	public void setMutipleEdit(Boolean mutipleEdit) {
		this.mutipleEdit = mutipleEdit;
	}

	public Boolean getBoolEdit() {
		return boolEdit;
	}

	public void setBoolEdit(Boolean boolEdit) {
		this.boolEdit = boolEdit;
	}

	public Boolean getFillBlankEdit() {
		return fillBlankEdit;
	}

	public void setFillBlankEdit(Boolean fillBlankEdit) {
		this.fillBlankEdit = fillBlankEdit;
	}

	public Boolean getCanStuDel() {
		return canStuDel;
	}

	public void setCanStuDel(Boolean canStuDel) {
		this.canStuDel = canStuDel;
	}

	public Boolean getEntranceExam() {
		return isEntranceExam;
	}

	public void setEntranceExam(Boolean entranceExam) {
		isEntranceExam = entranceExam;
	}

	public Boolean getIsEntranceExam() {
		return isEntranceExam;
	}

	public void setIsEntranceExam(Boolean isEntranceExam) {
		this.isEntranceExam = isEntranceExam;
	}

	public Boolean getIsFaceVerify() {
		return isFaceVerify;
	}

	public void setIsFaceVerify(Boolean isFaceVerify) {
		this.isFaceVerify = isFaceVerify;
	}

	public Integer getFaceVerifyStartMinute() {
		return faceVerifyStartMinute;
	}

	public void setFaceVerifyStartMinute(Integer faceVerifyStartMinute) {
		this.faceVerifyStartMinute = faceVerifyStartMinute;
	}

	public Integer getFaceVerifyEndMinute() {
		return faceVerifyEndMinute;
	}

	public void setFaceVerifyEndMinute(Integer faceVerifyEndMinute) {
		this.faceVerifyEndMinute = faceVerifyEndMinute;
	}

	public String getCheatingRemark() {
		return cheatingRemark;
	}

	public void setCheatingRemark(String cheatingRemark) {
		this.cheatingRemark = cheatingRemark;
	}

	public Boolean getShowCheatingRemark() {
		return showCheatingRemark;
	}

	public void setShowCheatingRemark(Boolean showCheatingRemark) {
		this.showCheatingRemark = showCheatingRemark;
	}

}
